/*
 * Project name:
     Hall current click (Using mikroE's Current click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release (FJ);
 * Description:
     This project is a simple demonstration of working with the Hall Current Click board
     based on the TLI4970 high precision miniature coreless magnetic current sensorwith SPI Interface.
     This example demosntrates measuring the current drawn by the consumer.
 * Test configuration:
     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev. Board:      EasyAVR7
                      http://www.mikroe.com/easyavr/
     Oscillator:      Oscillator Frequency 8.0000 MHz, 8.0000 MHz Crystal
     ext. modules:    hall current click board click
                      http://www.mikroe.com/click/hall-current/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
     - Place Hall current click board in the mikroBUS socket 1.
     - Turn on LCD backlight on SW3.1.
     - Turn on SW5.4, SW5.5 and SW5.6 (SPI).
     - Maximal SPI clock frequency for TLI4970 running at 3.3V is 5MHz.
 */

// Lcd module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

sbit Chip_Select           at PORTA5_bit;
sbit Chip_Select_Direction at DDA5_bit;

char txt[15];
int current_read;
float current_read_float;

// Get current value
unsigned Hall_Read() {
  // Local variable
  unsigned int tmp = 0, tmp0 = 0, tmp1 = 0;
  unsigned int buffer = 0;

  Chip_Select = 0;          // Select MCP3201
  tmp0 = SPI1_Read(buffer); // Get value
  tmp1 = SPI1_Read(buffer); // Get value
  Chip_Select = 1;          // Deselect MCP3201
  
  tmp = ((tmp1) | (tmp0 << 8));

  return tmp;               // Returns value
}

void main() {
  // Set chip select pin to be output
  Chip_Select_Direction = 1;
  Chip_Select = 1;                                              // Deselect MCP3201

  // Initialize SPI1 module
  SPI1_Init_Advanced(_SPI_MASTER, _SPI_FCY_DIV4, _SPI_CLK_LO_TRAILING);
  Delay_ms(100);

  // LCD Init
  LCD_Init();
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);
  
  // Display string on the LCD
  Lcd_Out(1,1,"Current [A] :");

  while(1) {
    // Read current from Hall Click
    current_read = Hall_Read();
    
    // If Sensor Current Value Transmission
    if ((current_read >> 15) == 0) {
      current_read = current_read & 0x1FFF;                     // Get [13:0] bits of current value
      current_read = current_read - 4096;                       // Map the current value
      current_read_float = (float)current_read * 0.0125;        // Calculate the current value in Amperes
      FloatToStr(current_read_float,txt);                       // Convert the current value to a string
      sprintf(txt, "%2.2f", current_read_float);                // Trim the string from leading spaces

      // Display the current value
      Lcd_Out(2,1,txt);
    }
    delay_ms(1000);
  }
}